'use strict';

const fs=require('fs');
const net=require('net');
const crypto=require('crypto');
const etools=require('ewt-etools');
const protobuf=require('protobufjs');
const protoRoot=protobuf.loadSync(__dirname+'/Plant3.proto');

// 函数集合
const funcDict={
  ip2raw:function (ipStr){
    // ipv6方式识别
    if(ipStr.includes(':')){
      const ipRaw=Buffer.allocUnsafe(16);
      const sp=ipStr.split(':');
      const ignLen=8-sp.length;
      let p=0;// 填充下标
      // 遍历分离数组
      for(let i=0;i<sp.length;i++){
        switch (sp[i].length){

        }
      }
    }
    // ipv4方式识别
    else {
      const numArry=ipStr.split('.').map(i=>{return Number.parseInt(i)});
      return Buffer.from(numArry);
    }
  },
  raw2ip:function (ipRaw){

  }
};

// 是否是调试模式
let isDebug=false;
process.execArgv.forEach(r=>{if(r.includes('debug')||r.includes('--inspect-brk=')) isDebug=true;});

// id合法性检验函数
function isIdError(id){
  if(typeof id!=='string') return true; // 必须是字符串
  if(id==='master') return true; // 不能是master
  if(id.length>40||id.length===0) return true; // 长度1~40
}

/* ====== 明文bufMsg消息处理 ====== */
// 构建bufMsg
function bufMsgBuild(buffer,typeCode){
  /* 输入校验 */
  // buffer类别
  if(!Buffer.isBuffer(buffer)) throw new Error('buffer is not Buffer');
  if(buffer.length>67108863) throw new Error('buffer too long, more then 64MB');
  // code必须是0～64之间
  if(!Number.isSafeInteger(typeCode)||typeCode<0||typeCode>63) throw new Error('code Error');
 /* 构建消息头 */
  const head=Buffer.allocUnsafe(8);

  // 0~3  低26位长度，高6位typeCode
  const b0_4=buffer.length+(typeCode<<26);
  head.writeUInt32LE(b0_4,0);
  // 4    随机位
  head[4]=Number.parseInt(Math.random()*256);
  // 5~7 异或校验码
  const checkNum=getCheckNum(buffer,b0_4+head[4]);
  head[5]=checkNum%256;
  head[6]=(checkNum>>>8)%256;
  head[7]=checkNum>>>16;

  // 合并head和buffer，即为输出的bufMsg
  return Buffer.concat([head,buffer]);
}
// 异或校验码计算，3字节
function getCheckNum(buffer,seed) {
  seed=seed%16000000;// 不超过2^24即可
  // 遍历buffer计算
  for(let i=0;i<buffer.length;i+=3){
    // 原始数字
    let num=((buffer[i]||0)<<16)+((buffer[i+1]||0)<<8)+(buffer[i+2]||0);
    // 循环位数
    const move=i%8;
    const high=num>>>move;
    const low=num-(high<<move);
    num=high+(low<<(24-move));

    seed=seed^num;
  }
  // 返回
  return seed;
}

// 二进制流片段缓存，应对分包粘包问题
/*
* 实例化时，需跟int参数限定最大允许消息体长度
*
* 接收：调用方法bufPieceIn传入
*
* 输出：
*   重新定义bufMsgFunc(bufMsg)方法，接收整理好的标准bufMsg
*   重新定义errFunc(errcode)方法，errcode可能列表如下：
*     overMaxLen    - 超出允许最大长度
*
* 触发errFunc或bufMsgFunc后，均会实例均会复位，可继续使用
* */
class c_fluentCache {
  constructor(maxLen){
    // 默认允许最大64MB
    maxLen=maxLen||67108863;
    // 如果最大长度设定有误，报错
    if(!Number.isSafeInteger(maxLen)||maxLen<1||maxLen>67108863) throw new Error('maxLen Error');
    // 最大允许长度
    this.maxLen=maxLen;
    // 头4位缓存，始终复用
    this.head=Buffer.allocUnsafe(4);
    // 对外接口函数，需在外部替换
    this.errFunc=(errcode)=>{}; // 错误触发函数
    this.bufMsgFunc=(bufMsg)=>{}; // 切开的消息体

    // 初次调用复位方法
    this.reSet();
  }
  // 复位
  reSet(){
    // 当前接收指针，p<4时，长度位未完整；p>4时，接收到body
    this.p=0;
    // 消息体积缓存，长度位接收完毕时创建
    this.msgBuf=null
  }
  // 二进制流片段进入
  bufPieceIn(bufPiece){
    // 验证输入，不是Buffer对象则报错
    if(!Buffer.isBuffer(bufPiece)) throw new Error('bufPiece is not Buffer');

    /* ========== 将二进制片段分析赋值到类内部缓存 ========= */
    for(let i=0;i<bufPiece.length;i++) {
      // 当长度位读取完毕时，构建缓存
      if(this.p===4) {
        // 消息体长度，前4字节的低26位为长度
        const bodyLen=this.head.readUInt32LE() & 67108863; // 相当于二进制的26个1
        // 超过允许最大长度，抛出错误
        if(bodyLen>this.maxLen){
          this.reSet(); // 复位
          this.errFunc('overMaxLen'); // 抛出错误
          break; // 退出循环
        }
        // 未超过，准备接收消息体
        else {
          // 构建消息缓存
          this.msgBuf=Buffer.allocUnsafe(8+bodyLen); // 前置码一共8位，前4位是长度位
          // 将长度位写入
          for(let i=0;i<4;i++) this.msgBuf[i]=this.head[i];
        }
      }


      // p<4时，读取长度位
      if(this.p<4) {
        this.head[this.p]=bufPiece[i];
        this.p++; // 指针位后移
      }
      // p>=4时，读取消息体
      else {
        this.msgBuf[this.p]=bufPiece[i];
        this.p++; // 指针位后移

        // 如果已经接收到最后，触发消息，并复位
        if(this.p===this.msgBuf.length) {
          this.bufMsgFunc(this.msgBuf); // 触发
          this.reSet(); // 复位
        }
      }
    }
  }
}

/* ====== 公用基类 ====== */

// protoWork类及实例
class c_protoWork{
  constructor(root){
    this._root=root;
  }

  /**
   * 将json序列化为指定格式
   * @param json 必须是字典
   * @param typ 指定的格式字符串
   */
  encode(json,typ){
    // json必须是字典
    if(etools.etype(json)!=='dict') throw new Error('json is not Dict');
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 确认格式
    const errMsg=protoObj.verify(json);
    if(errMsg) throw new Error('json format Error with type: '+typ+'  Json: '+JSON.stringify(json));
    // 实例化消息
    const mid=protoObj.create(json);
    // 取得流并反馈
    return protoObj.encode(mid).finish();
  }

  decode(buf,typ){
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 解析并反馈
    return protoObj.decode(buf);
  }

  valueEncode(obj,pArry){
    // 防嵌套数组，递归时传递，没有则新建
    if(!pArry) pArry=[];

    // 获取原型类别
    const objTyp=etools.protoType(obj);

    // 根据etype递归调用
    switch (objTyp) {
      case '[object Null]':return {typ:'0'};
      case '[object Undefined]':return {typ:'1'};
      case '[object Number]':
        if(Number.isSafeInteger(obj)) return {typ:'2',raw:this.encode({obj},'ProtoBuf.Plant3.v_int')};
        else return {typ:'3',raw:this.encode({obj},'ProtoBuf.Plant3.v_float')};
      case '[object Array]':
        const array=new Array(obj.length);
        for(let i=0;i<obj.length;i++)
          array[i]=this.valueEncode(obj[i],pArry);
        return {typ:'4',raw:this.encode({obj:array},'ProtoBuf.Plant3.v_array')};
      // buffer
      case '[object Uint8Array]':
        return {typ:'5',raw:this.encode({obj},'ProtoBuf.Plant3.v_bytes')};
      case '[object String]':
        return {typ:'6',raw:this.encode({obj},'ProtoBuf.Plant3.v_string')};
      case '[object Object]':
        // 类别优先使用指定的，若没有则使用原型名
        const protoTyp=obj._protoBufType||obj.constructor.name;
        // 普通对象
        if(protoTyp==='Object'){
          // 嵌套检查，有嵌套则报错
          if(pArry.includes(obj)) throw new Error('obj 嵌套错误');
          pArry.push(obj);

          // 递归调用
          const dictArry=[];
          for(const key in obj){
            // 忽略 __proto__项目
            if(key==='__proto__') continue;
            dictArry.push({key,val:this.valueEncode(obj[key],pArry)});
          }
          return {typ:'7',raw:this.encode({obj:dictArry},'ProtoBuf.Plant3.v_dict')};
        }
        // ProtoBuf对象
        else {
          // 不能删除 delete obj._protoBufType;
          return {typ:protoTyp,raw:this.encode(obj,protoTyp)};
        }
      case '[object Boolean]':
        // true false 分别占用一个单独的类型代码
        return {typ:obj?'8':'9'};
      default:
        throw new Error('unsport format: '+protoTyp);
    }
  }

  valueDecode(paramItem){
    switch (paramItem.typ){
      case '0':return null;
      case '1':return;// undefined
      case '8':return true;
      case '9':return false;
      case '2':
        const obj=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_int').obj;
        // 后台需要toNumber方法，前端不能执行
        return (typeof obj==='number')?obj:obj.toNumber();
      case '3':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_float').obj;
      case '4':
        // 先提取成数组
        const arry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_array').obj;
        // 再迭代还原
        return arry.map(r=>{return this.valueDecode(r)});
      case '5':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_bytes').obj;
      case '6':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_string').obj;
      case '7':
        // 先提取成数组
        const dictArry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_dict').obj;
        const result={};
        dictArry.forEach(r=>result[r.key]=this.valueDecode(r.val));
        return result;
      default:
        return this.decode(paramItem.raw,paramItem.typ);
    }
  }
}
const protoWork=new c_protoWork(protoRoot);

/**
 * api机制通讯类
 *
 * socket可以是后台的socket或者前端的ws
 * 需提前提供的方法：
 * sendProtoBuf - 发送二进制流，外层可继续封装
 * destroy - 销毁方法，销毁后内部的isSocketOk方法一定要能判定出销毁了
 */
class c_apiConn {
  constructor(socket,apiFunc,showName) {
    this.socket=socket;
    // 唯一函数处理方法，
    if(typeof apiFunc==='function') this.staticFunc=apiFunc;
    // 函数搜索类方法
    else this.apiFunc=apiFunc;
    this.showName=showName || 'apiConn';
    this.protoWork=protoWork;
    // 最后收到消息的时间戳，用于心跳，默认用当前时间
    this._lastRecSt=Date.now();

    // 调试开关，默认为关闭
    this.debug=false;
    // 对外调用count计数器
    this._count = 1;
    // 己方对外请求的缓存
    this._reqCache = {
      // key 发出时己方生成的count
      // value 数组三项：[sec, apiReqBuf, callback]
    };
    // 对方请求，己方运行完毕的结果缓存
    this._resCache = {
      // key 对方请求的count
      // value 数组三项：[sec, apiResBuf]
      // 其中，json项为null，则表明己方接收到请求但尚未处理完毕；为object时，表明已经处理完毕
    };
    // 内部秒循环
    this._secCircle =setInterval(()=>this._secCircleFunc(),1000);
    // 是否已被销毁
    this._destroyed=false;
  }

  // 秒循环工作函数
  _secCircleFunc() {
    /* 缓存定时清理 */
    // 响应结果保存30秒，超过时间删除
    for (const count in this._resCache) {
      // 获取发送对象
      const resObj = this._resCache[count];
      // 计时器累加
      resObj[0]++;
      // 超时删除
      if (resObj[0] >= 30) {
        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 缓存到期，删除，count: ` + count);

        delete this._resCache[count];
      }
    }

    // 请求收到反馈时已经删除，未收到时在缓存里
    for (const count in this._reqCache) {
      // 获取发送对象
      const reqObj = this._reqCache[count];
      // 计时器累加
      reqObj[0]++;
      // 第10、20秒重新发送
      if (this.isSocketOk() && (reqObj[0] === 10 || reqObj[0] === 20)) {
        this.socket.sendProtoBuf(reqObj[1]);

        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-REQ 重发请求，count: ` + count);
      }
      // 超过30秒删除，执行回调函数
      if (reqObj[0] >= 30) {
        delete this._reqCache[count]; // 从字典中删除
        reqObj[2] && reqObj[2]('timeout'); // 执行回调函数，报超时错误

        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-REQ 请求超时，count: ` + count);
      }
    }

    /* 心跳发送与检验 */
    const now = Date.now();
    // socket存在且最后收到消息时间超过30秒后，每秒都发送ping
    if (this.isSocketOk() && now - this._lastRecSt > 30e3) {
      // 组织请求体
      const apiReqJson = {
        count: this._getCount(),
        cmd: 'ping'
      };
      // 序列化
      const apiReqBuf = this.protoWork.encode(apiReqJson, 'ProtoBuf.Plant3.bufMsg_3_api');
      // 发送
      this.socket.sendProtoBuf(apiReqBuf);
    }
    // 超过40秒，销毁socket并设定为null
    if (this.socket && now - this._lastRecSt > 40e3){
      // socket和ws的关闭方法都尝试执行
      if(typeof this.socket.destroy==='function') this.socket.destroy();
      if(typeof this.socket.close  ==='function') this.socket.close();

      // 清除记录
      this.socket=null;
    }
  }

  // 判定socket是否可用
  isSocketOk() {
    // 获取通讯对象，可能是socket或ws
    const obj=this.socket;
    // 无对象
    if(!obj) return false;

    // socket 连接中
    if(obj.connecting===true) return false;

    // socket不可写或不可读
    if(obj.readable===false||obj.writable===false) return false;

    // 各种检测通过，反馈true
    return true;
  }

  // 获取请求序号
  _getCount(){
    // 获取序号
    const count = this._count;
    if (count < 214e7) this._count++; // 计数器累加
    else this._count = 1; // 越界归1

    return count;
  }

  // 接到对方调用API请求
  _rec_apiReq(apiReq){
    // 如果已经销毁，不再接收消息
    if(this._destroyed) return;

    // 如果有cmd，则特殊处理
    if(apiReq.cmd){
      // 构造基础反馈体
      const apiRes = {
        count: -apiReq.count, // 负数表示反馈
        cmd:''                // 反馈cmd，暂为空字符串
      };

      // 识别命令
      switch (apiReq.cmd){
        // 收到心跳ping，回pong
        case 'ping':
          apiRes.cmd='pong';
          break
      }

      // 组织反馈
      if(this.isSocketOk()){
        const apiResBuf=this.protoWork.encode(apiRes,'ProtoBuf.Plant3.bufMsg_3_api');
        this.socket.sendProtoBuf(apiResBuf);
      }

      return;
    }

    // 普通 API调用请求

    /* 根据count判断该此请求要怎么处理 */
    const resCache = this._resCache[apiReq.count];

    // 新请求，缓存字典中无记录，着手处理该请求
    if(!resCache){
      // 调试输出
      if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 新请求(${apiReq.api})，建立缓存，执行，count: `+apiReq.count);
      // 首先在resCache字典中，建立记录
      const resCache = this._resCache[apiReq.count]=[0,null,null];

      // 获取处理函数
      const func=this.staticFunc||this.apiFunc.getFunc(apiReq.api);

      // 构造info对象
      const info={
        socket:this.socket,
        api:apiReq.api,
        heads:{}
      };
      // 写入heads信息
      for(let i=0;i<apiReq.heads.length;i+=2) info.heads[apiReq.heads[i]]=apiReq.heads[i+1];

      // 执行
      func(info,apiReq.params.map(b=>this.protoWork.valueDecode(b)),(...all)=>{
        // 构造反馈体
        const apiRes = {
          count: -apiReq.count, // 负数表示反馈
          params:all.map(b=>this.protoWork.valueEncode(b))
        };

        // 将apiRes整体序列化
        const apiResBuf=this.protoWork.encode(apiRes,'ProtoBuf.Plant3.bufMsg_3_api');

        // 将反馈结果存入缓存
        resCache[1]=apiResBuf;

        // 判断连接状态
        if(this.isSocketOk()){
          // 将处理结果发送出去
          this.socket.sendProtoBuf(apiResBuf);
          // 调试输出
          if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 处理完毕，填充缓存，初次发送，count: `+apiReq.count);
        }
        // 无法发送
        else if (this.debug) {
          etools.log(`[${this.showName}] [DEBUG] API-RES 处理完毕，填充缓存，初次发送失败，count: `+apiReq.count);
        }
      });
    }
    // 已经接受到请求，且已经处理完毕，再次发送处理结果
    else if (resCache[1]){
      // 判断连接状态
      if(this.isSocketOk()){
        // 再次将处理结果发送出去
        this.socket.sendProtoBuf(resCache[1]);
        // 调试输出
        if (this.debug) etools.log('[Plant] [DEBUG] API-RES 重复请求，已处理完毕，再次发送，count: '+apiReq.count);
      }
      // 无法发送
      else if(this.debug){
        etools.log('[Plant] [DEBUG] API-RES 重复请求，已处理完毕，再次发送失败，count: '+apiReq.count);
      }
    }
    // 已经接受到请求，但尚未处理完毕，不用任何处理
    else {
      // 调试输出
      if (this.debug) etools.log('[Plant] [DEBUG] API-RES 重复请求，尚未处理完毕，不响应，count: '+apiReq.count);
    }
  }

  // 接到己方请求的反馈
  _rec_apiRes(apiRes){
    // 如果已经销毁，不再接收消息
    if(this._destroyed) return false;

    // 如果有cmd，则特殊处理
    if(apiRes.cmd){
      // 识别命令
      switch (apiRes.cmd){
        // 收到心跳pong，不做任何处理
        case 'pong':break
      }

      return;
    }

    // 从缓存取出请求对象
    const reqObj=this._reqCache[apiRes.count];
    // 取出成功
    if(reqObj){
      // 从缓存中删除该记录
      delete this._reqCache[apiRes.count];

      // 有回调函数，解析执行
      if(reqObj[2]){
        reqObj[2](...apiRes.params.map(b=>this.protoWork.valueDecode(b)));
      }

      // 调试输出
      if (this.debug) etools.log('[Plant] [DEBUG] API-REQ 请求响应成功，count: '+apiRes.count);
    }
    // 取出失败意味着已经超时，不用管了
  }

  // 接收protoBuf流
  recProtoBuf(buf){
    // 记录收到消息时间戳
    this._lastRecSt=Date.now();
    // 解析
    const apiObj = this.protoWork.decode(buf,'ProtoBuf.Plant3.bufMsg_3_api');
    // 请求处理
    if(apiObj.count>0) this._rec_apiReq(apiObj);
    // 反馈处理
    else{
      apiObj.count=-apiObj.count;
      this._rec_apiRes(apiObj);
    }
  }

  /**
   * 调用对方API请求
   *
   * @param apiObj string-API名；object-详细字典
   * @param params 参数数组，若为单个，则内部包装成数组
   * @param callback 回调函数，可以为空
   */
  callApi(apiObj,params,callback) {
    /* api请求信息解析 */
    let api,heads;
    // 字符串格式的支持
    if(typeof apiObj==='string'){
      heads={};
      api=apiObj;
    }
    // 对象形式的支持
    else {
      api=apiObj.api;
      heads=apiObj.heads||{};
    }

    // 请求参数处理，必须是数组
    if(!Array.isArray(params)) params=[params];

    // 包装请求体
    const reqJson={
      count:this._getCount(),
      api,
      heads:[],
      params:new Array(params.length)
    };

    // 写入请求头信息
    for(const key in heads){
      if(!heads.hasOwnProperty(key)) continue;// 跳过非属性项
      const value=heads[key];
      const typ=typeof value;
      let str;
      // 可传输的类型
      if(typ==='number'||typ==='boolean') str=value.toString();
      else if(typ==='string') str=value;
      else if(value===null) str='null';
      // 不可传输，跳过
      else continue;

      // 写入
      reqJson.heads.push(key,str)
    }

    // 写入参数
    for(let i=0;i<params.length;i++)
      reqJson.params[i]=protoWork.valueEncode(params[i]);

    // 生成流
    const apiReqBuf=this.protoWork.encode(reqJson,'ProtoBuf.Plant3.bufMsg_3_api');

    // 缓存请求
    this._reqCache[reqJson.count] = [0, apiReqBuf,callback];

    // 初次发送请求
    if(this.isSocketOk()){
      this.socket.sendProtoBuf(apiReqBuf);// 调试日志
      if(this.debug) etools.log('[Plant] [DEBUG] API-REQ 发出新请求，count: '+reqJson.count);
    }
    else if(this.debug){
      if(this.debug) etools.log('[Plant] [DEBUG] API-REQ 发出新请求失败，count: '+reqJson.count);
    }
  }

  // 销毁
  destroy() {
    // 标记已经销毁
    this._destroyed=true;
    // 清空定时器
    clearInterval(this._secCircle);
  }
}
// api机制处理程序注册查询类
class c_apiFunc {
  constructor(){
    // 对外提供的API字典集合（高优先级）
    this._apiDict = {};
    // 对外提供的API正则匹配数组（低优先级，有顺序）
    this._apiRegex = [];
  }
  // 设定对外提供的API
  reg(api, func) {
    // 根据api类型，来确定设定方法
    switch (etools.etype(api)){
      // 字符串：添加字典匹配方法
      case 'string':
        // 转为字典形式，不能break
        const obj = {};
        obj[api] = func;
        api = obj;

      // 字典：key为api名，value为处理方法，依次添加到内部字典
      case 'dict':
        // 判定api必须为字典，否则为编程错误
        if (etools.etype(api)!=='dict') throw new Error('apiName format Error');

        // 遍历字典
        for (const key in api) {
          const func = api[key];
          // 验证func必须是函数，否则是编程错误
          if (etools.etype(func)!== 'function') throw new Error('func format Error');
          // 内部登记
          this._apiDict[key] = func;
        }
        break;

      // 正则对象，添加到正则匹配数组
      case 'regex':
        // 包装成数组，不Break
        api=[[api,func]];

      // 数组：有序多个正则匹配方法
      case 'array':
        // 遍历数组添加
        for(let i=0;i<api.length;i++){
          const item=api[i];
          // 验证api项目必须是正则对象
          if(etools.etype(item[0])!=='regex') throw new Error('regex format Error');
          // 验证func必须是函数，否则是编程错误
          if (etools.etype(item[1])!== 'function') throw new Error('func format Error');
          // 验证成功，添加到内部字典
          this._apiRegex.push(item);
        }
        break;

      // 未识别
      default: throw new Error('unknow how to regApi');
    }
  }

  // 获取方法函数
  getFunc(api){
    // api必须是字符串，否则为编程错误
    if(typeof api!=='string') throw new Error('api must be String');

    // 优先级 1：如果有字典匹配，直接反馈
    if(api in this._apiDict) return this._apiDict[api];

    // 优先级 2：依次尝试正则匹配，匹配成功，则反馈
    for(let i=0;i<this._apiRegex.length;i++)
      if(this._apiRegex[i][0].test(api)) return this._apiRegex[i][1];

    // 末次：返回未找到方法
    return ((info,[], callback)=>{callback('noApi: ' + api)})
  }
}
// 事件机制类，依赖于api体系
class c_event{
  constructor(netPoint){
    this.debug=true;
    // 事件值获取函数
    this._netPoint=netPoint;
    // 事件值字典，与是否订阅无关
    this._eventValues={};

    // 对方订阅请求
    netPoint.apiFunc.reg('_eventBook',(...all)=>this._api_eventBook(...all));
    // 对方触发事件
    netPoint.apiFunc.reg('_eventEmit',(...all)=>this._api_eventEmit(...all));

    // 己方向外订阅字典
    /*
    * key为event名称
    * 值字典中：
    *   funcs      - 数组，订阅函数集合
    *   paramsArry - 订阅结果数组，既最近emit数组，若为null则表明之前没有emit过
    * */
    this._books={};

    // 外方向己方订阅的事件名字典
    /*
     * key为事件名
     * 值为订阅方的id数组
     */
    this._emits={};

    /**
     * 订阅验证函数
     *
     * 默认在err位置返回false,，即允许任何订阅
     * @param info
     * @param post
     * @param callback
     */
    this.bookCheck=(eventName,id,callback)=>{
      callback();
    };
    /**
     * 彻底取消订阅，即该事件没有任何id在订阅
     */
    this.cancel=(eventName)=>{}
  }

  // 订阅请求处理
  _api_eventBook(info,[eventName],callback){
    const id=info.socket.id;
    // 调用验证函数，判定是否允许订阅
    this.bookCheck(eventName,id,(err)=>{
      // 如果有err项目，表明不允许订阅
      if(err) callback(err);
      // 允许订阅
      else {
        callback(); // 空反馈表示订阅成功

        etools.log(`来自${id}对${eventName}的新订阅`);

        /* 记录订阅 */
        // emit字典中没有数组，创建
        if(!this._emits[eventName]) this._emits[eventName]=[id];
        // 有数组，但未包含该id，添加
        else if(!this._emits[eventName].includes(id)) this._emits[eventName].push(id);

        /* 首次推送 */
        // 如果当前字典中有值，进行一次推送
        const paramsArry=this._eventValues[eventName];
        if(paramsArry) info.socket.callApi('_eventEmit',[eventName,paramsArry],()=>{});
      }
    })
  }

  // 对方触发事件
  _api_eventEmit(heads,[eventName,paramsArry],callback){
    const event=heads.socket.id+'.'+eventName;
    const bookObj=this._books[event];
    // 如果己方没有订阅，则callback错误
    if(!bookObj) callback('noBook');
    // 己方有订阅
    else {
      // 记录本次推送的结果
      bookObj.paramsArry=paramsArry;
      // 取当前funcs数组长度
      const len=bookObj.funcs.length;
      // 依次从头取出，执行反馈true，则push到最后，否则不push
      for(let i=0;i<len;i++){
        // 取出
        const func=bookObj.funcs.shift();
        // 执行，若反馈为true，则再添加到数组末尾
        if(func(...paramsArry)) bookObj.funcs.push(func);
      }
      // 如果最终数组长度大于0，空反馈，即继续订阅
      if(bookObj.funcs.length>0) callback();
      // 没有func了，反馈err取消订阅
      else {
        delete this._books[event];
        callback('noBook');
      }
    }
  }

  // 服务列表更新，刷新订阅状态
  _reServicesInfo(){
    // 遍历所有当前订阅的事件
    for(const id in this._books){
      const bookObj=this._books[id];
      const service=this._netPoint.servicesInfo[bookObj.serviceId];
      // 如果服务不在线，标记state为-2
      if(!service) bookObj.state=-2;
      // 服务在线，订阅不正常，尝试重新订阅
      else if(bookObj.state<1) this._bookCall(bookObj);
      // 服务在线，但state不等于当前的regSt，无论订阅状态均需要重新订阅
      else if(service.regSt!==bookObj.state) this._bookCall(bookObj);
    }
  }

  // 向对应服务发送订阅请求，并更新内部标记状态
  _bookCall(bookObj){
    // 调用api机制向对方订阅
    this._netPoint.callApi(bookObj.serviceId+'._eventBook',bookObj.eventName,(err)=>{
      // 订阅错误标记
      if(err){
        bookObj.state=-1;
        bookObj.err=err;
      }
      // 订阅成功：前端，无this._netPoint.servicesInfo，state记录为1
      else if(!this._netPoint.servicesInfo) bookObj.state=1;
      // 订阅成功：后端，state记录为服务对应的regSt
      else bookObj.state=this._netPoint.servicesInfo[bookObj.serviceId].regSt;
    })
  }

  // 对外方法：订阅
  book(event,func){
    // 事件名非字符串，程序编写错误
    if(!event||typeof event!=='string') throw new Error('eventName must be string and not empty');
    // 订阅函数类型
    if(typeof func!=="function") throw new Error('func typeError');

    // 尝试读取已有订阅信息
    let bookObj=this._books[event];
    // 已经订阅，无论订阅状态
    if(bookObj){
      // 缓存func
      bookObj.funcs.push(func);
      // 如果之前曾推送过结果，对本func触发一次
      if(bookObj.paramsArry) func(undefined,...bookObj.paramsArry);
    }
    // 新订阅
    else {
      // 分离服务名称和事件名称
      const index=event.indexOf('.');
      if(!index>0) throw new Error('fullEventName: serviceId + eventName');
      const serviceId=event.substr(0,index);
      const eventName=event.substr(index+1);

      // 新建事件本地存储，paramsArry为空
      bookObj=this._books[event]={
        serviceId,eventName,
        state:0,        // 0 初次订阅中；-1 订阅错误；-2 相应服务下线
                        // state 为正数时，记录订阅成功时该服务的regSt
        err:null,       // 最近一次的错误信息，重新订阅成功时不清除
        funcs:[func],   // 订阅的func数组
        paramsArry:null // 最后一次推送的订阅信息
      };
      // 调用api机制向对方订阅
      this._bookCall(bookObj);
    }
  }

  // 触发某事件的推送，值需要从外部字典获取，若无值，则不必继续
  emit(eventName,...paramsArry){
    // 记录值，为以后新订阅初次发送准备
    this._eventValues[eventName]=paramsArry.length>0?paramsArry:null;
    // 判定该事件是否被订阅，未订阅则结束
    const arry=this._emits[eventName];
    if(!arry) return;
    // 并行推送
    this._emits[eventName].forEach(id=>{
      this._netPoint.callApi(id+'._eventEmit',[eventName,paramsArry],(err)=>{
        // 推送无错误，可结束
        if(!err) return;
        // 推送有错误，判定错误原因

        // 取消订阅
        const arry=this._emits[eventName];
        const index=arry?arry.indexOf(id):-1;
        if(index>=0) arry.splice(index,1);
        if(this.debug) etools.log('订阅取消：'+eventName +' @ '+id);

        // 订阅为空，触发cancel方法
        if(arry && arry.length===0){
          delete this._emits[eventName];
          this.cancel(eventName);
          if(this.debug) etools.log('订阅彻底取消：'+eventName);
        }
      })
    });
  }
}

// 通讯基础类，每个socket建立一个
class c_econn{
  constructor(socket,apiFunc){
    /* 消息接收 */
    this.socket=socket;
    // 分配分包粘包类
    const cache=new c_fluentCache(); // 默认限制极大允许64兆
    // API机制类
    this.api=new c_apiConn(socket,apiFunc,'Plant');
    // this.api.debug=true;
    socket.on('error',(err)=>{
      etools.log('连接断开')
    });
    socket.on('close',()=>{
      this.api.destroy();
    });
    // TCP层数据接收
    socket.on('data',(buf)=>{
      cache.bufPieceIn(buf);
    });
    // 分包粘包输出bufMsg
    cache.bufMsgFunc=(bufMsg)=>{
      this._recBufMsg(bufMsg);
    };
    // 分包粘包出错，关闭socket
    cache.errFunc=(err)=>{
      socket.destroy();
    };
    // 增加发送bufMsg函数
    socket.sendBufMsg=(buf,typeCode)=>{
      if(typeCode>=24) throw new Error('sendBufMsg方法只能用于发送明文消息');
      // 如果socket已经销毁，不再继续
      if(socket.destroyed || !socket.writable) return;

      // 明文包装
      let bufMsg=bufMsgBuild(buf,typeCode);
      // 如果有证书，进行加密
      if(socket.certBuf){
        const key=Buffer.concat([socket.skeyBuf,socket.certBuf]);
        let cryptBuf;
        try{
          let cip = crypto.createCipher('aes-128-ecb', key);
          cryptBuf = Buffer.concat([cip.update(bufMsg),cip.final()]);
        } catch (e){
          // 遇到错误，销毁链接
          socket.destroy();
          return;
        }
        // 加密包装
        bufMsg=bufMsgBuild(cryptBuf,24);
      }

      // 发送
      this.socket.write(bufMsg);
    };
    // 增加发送protoBuf方法
    this.socket.sendProtoBuf=(buf)=>this.socket.sendBufMsg(buf,3);
    // 增加调用API方法
    this.socket.callApi=(...all)=>this.api.callApi(...all);
  }
  // 1、接收到bufMsg
  _recBufMsg(bufMsg){
    // 校验码比对
    const b0_4=bufMsg.readUInt32LE(0);
    const buf=bufMsg.slice(8);
    const jsNum=getCheckNum(buf,b0_4+bufMsg[4]);// 计算校验码
    const csNum=bufMsg[5]+(bufMsg[6]<<8)+(bufMsg[7]<<16);// 传输校验码
    // 若比对错误，销毁socket
    if(jsNum!==csNum){
      this.socket.destroy();
      return
    }
    // 分离typeCode，并分类处理
    const typeCode=b0_4>>>26;
    switch(typeCode){
      case 3: // 明文：API机制消息，使用protobuf
        this.api.recProtoBuf(buf);
        break;
      case 24: // 加密包，解密后迭代
        // 如果本地socket没有设定加密证书，直接断开
        if(!this.socket.certBuf){
          etools.log('socket 没有设定证书，但收到了加密消息，连接断开');
          this.socket.destroy();
        }
        // 有证书，可以进行解密
        else {
          const key=Buffer.concat([this.socket.skeyBuf,this.socket.certBuf]);
          const decipher = crypto.createDecipher('aes-128-ecb', key);
          let newbuf;
          try{
            const buf_update=decipher.update(buf);
            const buf_final =decipher.final();
            newbuf=Buffer.concat([buf_update,buf_final]);
          } catch (e) {
            etools.log('socket 解密过程出错，连接断开');
            this.socket.destroy();
            return;
          }
          // 解密完成，迭代
          this._recBufMsg(newbuf);
        }
        break;
      case 25: // gzip压缩包
        debugger;
        break;
      default:
        socket.emit('unknowBufMsg',bufMsg);
        break;
    }
  }
}

// 网点基础类
class c_netPoint{
  constructor(){
    // 对外监听实例，0 - 未建立,1 - 建立过程中
    this._tcpListener=0;
    // 已经连接的socket字典
    this.socketDict={
      /**
       * 己方为服务端时：
       *  在外部注册名为"serviceReg"的API，自行管理该字典内容
       */
      /**
       * 己方为客户端时：
       *  使用ConnectCore、ConnectMate方法连接后自动添加
       */
      // 使用ConnectCore方法时，己方自行添加
      // 己方作为core时，可在外部管理该字典
      //
    };
    // 定义自己的api字典
    this.apiFunc=new c_apiFunc();
    // 关联内部API：_connReg
    this.apiFunc.reg('_connReg',(...a)=>this._connReg.apply(this,a));
    /**
     * 建立内部事件对象
     *
     * master 需要在_publishServiceList方法，
     * service需要在API：_servicesInfo中，
     * 调用event的_reServicesInfo来进行订阅状态维护
    */
    this.event=new c_event(this);
  }

  // 监听初始化及其事件
  _serverInit(server,callback){

  }

  // 创建tcp链接，己方不作为core时可用，仅负责创建、连接，连接成功即可callback
  _socketTcpTry(port,host,callback){
    // 如果己方为核心，则不可以发起对外连接
    if(this.id==='master'){
      callback('self is Master, cannot connect out');
      return;
    }

    let runCallback=!callback;
    const socket=net.connect({port,host});
    socket.on('connect',()=>{
      if(!runCallback){
        runCallback=true;
        socket.removeAllListeners(socket.eventNames());
        callback(undefined,socket);
      }
    });
    socket.on('error',(err)=>{
      if(!runCallback){
        runCallback=true;
        socket.removeAllListeners(socket.eventNames());
        callback(err);
      }
    })
  }

  // 创建TCP监听
  listenTCP(port,host,callback){
    // 缺项目填充
    if(typeof port==='function'){
      callback=port;
      port=undefined;
      host='0.0.0.0';
    }
    else if(typeof host==='function'){
      callback=host;
      host='0.0.0.0';
    }
    else if(!host) host='0.0.0.0';
    if(!callback) callback=()=>{};

    // 根据当前状态决定动作
    switch (this._tcpListener){
      // 未监听，可执行后续动作
      case 0:break;
      // 建立监听中，报错
      case 1:
        callback('start listening');
        return;
      // 现有监听实例
      default:
        callback('tcpListing now');
        return;
    }

    // 标记连接建立中
    this._tcpListener=1;

    const server=net.createServer();   // 创建实例
    server.listen(port,host);          // 开启监听

    // 是否曾经执行过回调函数
    let runCallback=false;

    server.on('close',()=>{
      etools.log('netPoint Listen Close');
    });
    // 新连接到达，调用socket初始化方法
    server.on('connection',(socket)=>{
      // 实例化基础通讯
      socket.econn=new c_econn(socket,this.apiFunc);
    });
    // 监听错误
    server.on('error',(...all)=>{
      // 调试日志输出：
      if(isDebug){
        etools.log(`[Plant.netPoint] tcpListen: Self: ${this.id} Error`);
      }
      // 标记为未监听
      this._tcpListener=0;
      // 未执行callback则执行
      if(!runCallback){
        runCallback=true;
        callback.apply(this,all);
      }
    });
    // 监听成功
    server.on('listening',()=>{
      // 调试日志输出：
      if(isDebug){
        const address=server.address();
        etools.log(`[Plant.netPoint] tcpListen: Self: ${this.id} `+JSON.stringify(address));
      }
      // 记录监听实例
      this._tcpListener=server;
      // 未执行callback则执行
      if(!runCallback){
        runCallback=true;
        // 延迟100ms执行，等待其他异步程序
        callback.apply(this,[]);
      }
    })
  }

  // 根据具体id调用API
  callApi(apiObj,params,callback){
    // 允许以字符串，只提供api
    if(typeof apiObj==='string') apiObj={
      api:apiObj
    };
    // 允许没有params
    if(typeof params==='function'){
      callback=params;
      params=[];
    }
    // params必须是数组
    else if(!Array.isArray(params)) params=[params];
    // 允许没有callback
    if(!callback) callback=()=>{};

    // 从完整api中分离
    const n=apiObj.api.indexOf('.');
    if(n<0){
      // 用该接口必须指定服务名
      callback('api 中必须指定微服务的名称');
      return;
    }
    // 取出serviceId
    const serviceId=apiObj.api.substr(0,n);
    // 分离剩下的api部分
    apiObj.api=apiObj.api.substr(n+1);

    // 如果是调用自身，则简易处理
    if(serviceId===this.id){
      const apiFunc=this.apiFunc.getFunc(apiObj.api);
      apiObj.isSelf=true;
      // 执行
      apiFunc(apiObj,params,callback);
      // 不再继续
      return;
    }

    /* 根据当前socket字典情况，分别处理 */
    const socket=this.socketDict[serviceId];
    // 获取到，可以直接调用
    if(socket) socket.callApi(apiObj,params,callback);
    // 如果自己是master，至此可以放弃
    else if(this.id==='master') callback('no serviceId: '+serviceId);
    // 若有请求缓存，则添加到缓存队列
    else if(this.apiReqCache[serviceId]) this.apiReqCache[serviceId].push([apiObj,params,callback]);
    // 字典中有该服务，当前未连接，尝试连接
    else if(serviceId in this.servicesInfo) {
      // 先创建调用缓存
      this.apiReqCache[serviceId]=[[apiObj,params,callback]];
      // 再异步进行连接
      this.connService(serviceId,(err,socket)=>{
        // 无论成功或失败，需要将缓存中所有请求都执行了
        this.apiReqCache[serviceId].forEach(all=>{
          const callback=all[2];
          // 获取连接错误
          if(err||!socket){
            if(callback) callback(err||'build conn to Service Error: '+serviceId);
          }
          // 获取成功
          else socket.callApi(...all);
        });
        // 删除缓存
        delete this.apiReqCache[serviceId];
      })
    }
    // 无socket，无缓存，服务字典中也没有该id，放弃
    else callback('no serviceId: '+serviceId);
  }
}

// 控制核心
/**
 * conf参数：
 *    port - 监听端口
 *
 */
class c_master extends c_netPoint{
  constructor(conf){
    const port = conf.port;
    // 如果port不合格，抛错
    if(!Number.isSafeInteger(port)||port<1||port>65535) throw new Error('port illegal');
    super();

    // 内部变量
    this.id='master'; // 核心id必须是master

    // 连接认证函数
    /**
     * post字典：
     *  id: 标识
     *  certMd5: 证书md5编码，可能为空
     *  其他项记录字典，可扩展
     *
     * json反馈对象：
     *  reHost: 转向登录的主机
     *  rePort: 转向登录的端口，两者有其一，则其他项忽略
     *  sendCert: 是否向客户端发送证书
     *  其他项callback发送，不处理
     *
     * cert证书：有此项则为加密通讯
     */
    this.regFunc=(post,callback)=>callback();//证书查询，此处默认不需要证书，即明文通讯

    // 开启tcp监听
    this.listenTCP(port);
  }


  // 更新服务列表并向所有终端报告
  _publishServiceList(){
    // 如果设定不广播，则不执行任何动作
    if(!this.issueDict) return;
    const nowServices=[]; // 仅日志输出用
    // 遍历服务发送，其中master的ip是根据不同的socket单独变换
    for(const id in this.socketDict){
      nowServices.push(id);
      const socket=this.socketDict[id];

      const sendObj=etools.clone(this.servicesInfo);

      const host=socket.localAddress;
      // 单独变换master的host信息
      sendObj[this.id]=this._getRegObj(host);
      // 单独定义master的regSt为其自己的注册时间
      sendObj[this.id].regSt=this.servicesInfo[id].regSt;
      // 下发在线字典
      this.socketDict[id].callApi('_servicesInfo',sendObj,()=>{})
    }
    etools.log('[Plant] Service Publish: '+nowServices.join(', '));

    // 更新订阅信息状态
    this.event._reServicesInfo();
  }

  // TCP连接到达，内部API：_connReg
  _connReg(info,[post],callback){
    // 先调用外部regFunc
    this.regFunc(post,(err,json,cert)=>{
      // 如果有证书，但不合格，throw错误
      if(cert&&(!Buffer.isBuffer(cert)||cert.length!==96))
        throw new Error('cert Format Error, must be buffer and len = 96');

      // 外部认证错误，关闭连接，结束
      if(err){
        info.socket.destroy();
        return
      }
      // json不能为空
      json=json || {};

      // 无错误，但要求转向，直接反馈json并结束
      if(json.reHost||json.rePort){
        // 反馈转向信息
        callback(undefined,json);
        // 1秒后关闭该socket
        setTimeout(()=>info.socket.destroy(),1000);
        return;
      }

      // 生成会话随机密码，无论是否加密通讯，均会发送给客户端
      const skey=info.socket.skeyBuf=crypto.randomBytes(32);

      // 将是否加密写入反馈json
      json.crypto=!!cert;

      /* 反馈客户端，是否加密均适用 */
      // 发送证书的情况
      if(cert && json && json.sendCert) callback(undefined,json,skey,cert);
      // 不发送证书的情况
      else callback(undefined,json,skey);

      /* 有证书时，服务器端设定加密 */
      if(cert){
        info.socket.certBuf=cert;
        etools.log(`service(${post.id}) 进入加密模式`);
      }

      const id=post.id;
      /* ==== 重复id处理 ==== */
      // 踢下
      if(this.socketDict[id]) this.socketDict[id].destroy();

      /* ======= 注册 ======= */

      // 在socket字典中记录该连接
      this.socketDict[id]=info.socket;
      // 在服务字典中记录信息
      this.servicesInfo[id]=post;
      // 写入时间戳版本
      this.servicesInfo[id].regSt=Date.now();
      // 写日志
      etools.log(`[Plant] [Master] Service On : ${id}`);
      // 进行广播
      this._publishServiceList();
      // socket中记录id
      info.socket.id=id;

      /* ======= 注销 ======= */
      info.socket.on('close',()=>{
        // 如果字典中记录的就是本实例，则删除
        if(this.socketDict[id]===info.socket){
          delete this.socketDict[id];
          delete this.servicesInfo[id];
        }
        // 写日志
        etools.log(`[Plant] [Master] Service Off: ${id}`);
        // 通知更新
        this._publishServiceList();
      })
    });
  }
}

// 服务
/**
 * conf参数：
 *    id   - 自己的id名
 *    host - 控制核心的主机名
 *    port - 控制核心的端口
 *    certPath - 己方证书的路径
 */
class c_service extends c_netPoint{
  constructor(conf){
    // id必须是合法字符串
    if(isIdError(conf.id)) throw new Error('id illegal');
    // 如果port不合格，抛错
    if(!Number.isSafeInteger(conf.port)||conf.port<1||conf.port>65535) throw new Error('port illegal');

    super();

    // 己方的id，证书路径
    this.id=conf.id;
    this.certPath=conf.certPath;

    // 核心服务id，默认没有
    this._masterPort=conf.port;
    this._masterHost=conf.host;


    this._connMaster();

    // 重连计数器，及秒循环
    this._reConnCount=1;
    this._reConnNum=0;// 实际数据为2^reConnNum次方，最大为8
    this._secCircle=setInterval(()=>{
      // 当前连接正常
      if(this.socketDict.master){
        this._reConnCount=1;
        this._reConnNum=0;
      }
      // 当前连接不正常，但没有达到重连时间
      else if(this._reConnCount<Math.pow(2,this._reConnNum)){
        this._reConnCount++;
      }
      // 达到需要重连的时候
      else {
        // 计时器清零
        this._reConnCount=1;
        // 重连次数记录
        if(this._reConnNum<8) this._reConnNum+=0.2;
        this._connMaster()
      }
    },1000);
  }

  // TCP连接到达，注册内部API：_connReg
  _connReg(head,[post],callback){
    const id=post.id;

    // 验证该id在自己的信息列表中
    if(id in this.servicesInfo){
      // 登记
      this.socketDict[id]=head.socket;
      head.socket.id=id;
      // 写日志
      etools.log('[Plant] Connect From Service On: '+id);
      // 反馈成功
      callback();
      // 注销
      head.socket.on('close',()=>{
        if(this.socketDict[id]===head.socket){
          delete this.socketDict[id];
          // 写日志
          etools.log('[Plant] Connect From Service Off: '+id);
        }
      })
    }
    // 不在，销毁
    else {
      // 反馈失败，销毁该连接
      callback('regError');
      // 销毁该连接
      process.nextTick(()=>head.socket.destroy())
    }
  }

  // 更新服务字典
  _servicesInfo(head,[post],callback){
    this.servicesInfo=post;
    // 写日志
    etools.log('[Plant] Receive Service List: '+Object.keys(post).join(', '));

    // 更新订阅信息状态
    this.event._reServicesInfo();

    callback();
  }

  // 创建到核心的连接，连接到核心时需指定端口、主机
  _connMaster(rePort,reHost){
    const port=rePort||this._masterPort;
    const host=reHost||this._masterHost;

    // 如果到当前连接正常，不继续运行
    if(this.socketDict.master) return;

    // 创建tcp连接，转向登录优先
    this._socketTcpTry(port,host,(err,socket)=>{
      // 建立错误，无动作
      if(err){
        callback(err);
        return
      }

      // 实例化通讯基础类
      socket.econn=new c_econn(socket,this.apiFunc);
      // socket的id设置为master
      socket.id='master';

      // 自己的IP地址
      const host=socket.localAddress;

      /* 整理注册信息，模板：regToMaster_req */
      const regObj=protoRoot.lookupType('ProtoBuf.Plant3.regToMaster_req').create();
      // id标识
      regObj.id=this.id;
      // ip地址记录
      regObj.ipRawService=funcDict.ip2raw(socket.localAddress);
      regObj.ipRawMaster =funcDict.ip2raw(socket.remoteAddress);
      // 尝试读取本地证书并进行md5运算
      let localCert;
      if(this.certPath){
        try{
          localCert=fs.readFileSync(this.certPath);
          regObj.certMd5=etools.md5(localCert);
        } catch (e) {
          etools.log('读取本地证书或计算MD5失败');
        }
      }


      // 尝试向核心类注册
      socket.callApi('_regToMaster',regObj,(err,json,skey,cert)=>{
        // 连接到核心失败时，需断开本连接
        if(err){
          socket.destroy();
          // 执行回调
          callback(err);
          return;
        }
        // 如果要求转向登录
        if(json.rePort||json.reHost){
          etools.log(`转向登录到 port: ${json.rePort}, host:${json.reHost}`);
          // 断开本连接
          socket.destroy();
          // 迭代本方法
          setTimeout(()=>{
            this._connMaster(callback,json.rePort,json.reHost);
          },200);
          return;
        }

        // 如果服务器下发了新证书，且己方有存储路径，进行持久化
        if(cert && this.certPath){
          fs.writeFile(this.certPath,cert,(err)=>{
            if(err) etools.log('通讯证书写磁盘持久化失败，下次连接很可能会失败')
          });
        }

        // 如果要求加密，设定通讯加密方法
        if(json.crypto){
          const certEnd=cert||localCert; // 优先使用服务器下发，其次使用本地读取
          if(!certEnd) throw new Error('服务器要求加密，但未下发新证书，本地证书读取也失败');
          // 设定通讯加密证书
          etools.log(`service(${this.id}) 进入加密模式`);
          socket.certBuf=certEnd;
          socket.skeyBuf=skey;
        }

        // 成功，登记
        this.socketDict.master=socket;
        // 写日志
        etools.log(`[Plant] Connect Master On`);
        // 执行回调
        callback(undefined,socket);

        // 注销
        socket.on('close',()=>{
          if(this.socketDict.master===socket){
            this.socketDict.master=null;
            etools.log(`[Plant] Connect Master Off`);
          }
        })
      });
    })
  }

  // 创建到服务的连接，检索字典以获取端口、主机
  connService(id,callback){
    // 如果id已经存在，则拒绝
    if(id in this.socketDict){
      callback('id exist');
      return;
    }
    // 如果服务字典中没有相关信息，拒绝
    const info=this.servicesInfo[id];
    if(!info){
      callback('no info of id: '+id);
      return;
    }

    // 尝试建立    
    const tryBuild=(n)=>{
      // 如果n已经大于该服务监听端口数量，不在尝试
      if(n>=info.tcpListens.length){
        callback('cannot build connect');
        return;
      }
      // 取出当前端口和主机
      const m=info.tcpListens[n].lastIndexOf(':');
      const host=info.tcpListens[n].substr(0,m);
      const port=Number.parseInt(info.tcpListens[n].substr(m+1));

      // 尝试连接
      this._socketTcpTry(port,host,(err,socket)=>{
        if(err) tryBuild(n+1); // 尝试下一个
        else {
          // 通讯基础类
          socket.econn=new c_econn(socket,this.apiFunc);
          // 尝试向对方注册
          socket.callApi('_connReg',{id:this.id},(err)=>{
            // 连接到核心失败时，需断开本连接
            if(err){
              socket.destroy();
              tryBuild(n+1); // 尝试下一个
            }
            // 连接成功时，登记该id
            else {
              this.socketDict[id]=socket;
              socket.id=id;
              callback(undefined,socket);
              // 写日志
              etools.log('[Plant] Connect  to  Service On : '+id);
              // 注销
              socket.on('close',()=>{
                if(this.socketDict[id]===socket){
                  delete this.socketDict[id];
                  etools.log('[Plant] Connect  to  Service Off: '+id);
                }
              })
            }
          })
        }
      })
    };

    // 依次进行尝试
    tryBuild(0);
  }

  // 调用主程序api的包装命令
  masterApi(api,...all){
    this.callApi('master.'+api,...all);
  }
}

// 对外输出
module.exports={
  isDebug,
  c_fluentCache,
  c_master,
  c_service,
  c_apiConn,
  c_econn,
  c_apiFunc,
  c_event,
  // 添加proto文件
  addProtoFile(filepath){
    protoRoot.loadSync(filepath);
  },
  /**
   * 将proto对象转json方法
   */
  protoObj2jsonObj(obj){
    switch (etools.protoType(obj)) {
      case '[object Null]':
      case '[object RegExp]':
      case '[object Uint8Array]':
      case '[object Undefined]':
      case '[object Boolean]':
      case '[object Number]':
      case '[object String]': return obj;
      case '[object Array]': return obj.map(i=>{return protoObj2jsonObj(i)});
      case '[object Object]':
        // 本就是jsonObj，原样返回
        if(obj.constructor.name==='Object') return obj;

        // Long处理
        if(obj.constructor.name==='Long') return obj.toNumber();

        // 构造新jsonObj
        const jsonObj={
          _protoBufType:obj.constructor.name
        };
        // 遍历赋值
        for(const key in obj) jsonObj[key]=protoObj2jsonObj(obj[key]);

        // 反馈
        return jsonObj;
    }
  }
};